#!/usr/bin/env node

// src/index.ts
import { cac } from "cac";
import cpy from "cpy";
import { execa } from "execa";
import fs2 from "fs-extra";
import { oraPromise } from "ora";
import pico3 from "picocolors";
import { default as prompts } from "prompts";

// package.json
var package_default = {
  name: "create-wagmi",
  description: "Create wagmi apps with one command",
  version: "0.0.0-20221113003027",
  type: "module",
  license: "MIT",
  bin: {
    "create-wagmi": "dist/index.js"
  },
  scripts: {
    build: "tsup",
    "changeset:release": "pnpm build && changeset publish",
    "changeset:version": "changeset version && pnpm install --lockfile-only",
    dev: "tsup --watch",
    lint: "eslint . --cache",
    "lint:fix": "pnpm lint --fix",
    "lint:format": "prettier --write",
    prepare: "npx simple-git-hooks",
    typecheck: "tsc --noEmit"
  },
  dependencies: {
    cac: "^6.7.14",
    cpy: "^9.0.1",
    "cross-spawn": "^7.0.3",
    "detect-package-manager": "^2.0.1",
    execa: "^6.1.0",
    "fs-extra": "^10.1.0",
    ora: "^6.1.2",
    picocolors: "^1.0.0",
    prompts: "^2.4.2",
    "update-check": "^1.5.4",
    "validate-npm-package-name": "^4.0.0"
  },
  devDependencies: {
    "@changesets/changelog-github": "^0.4.4",
    "@changesets/cli": "^2.22.0",
    "@types/cross-spawn": "^6.0.2",
    "@types/eslint": "^8.4.6",
    "@types/fs-extra": "^9.0.13",
    "@types/node": "^18.11.7",
    "@types/prompts": "^2.4.1",
    "@types/validate-npm-package-name": "^4.0.0",
    "@typescript-eslint/eslint-plugin": "^5.37.0",
    "@typescript-eslint/parser": "^5.37.0",
    eslint: "8.17.0",
    "eslint-config-prettier": "^8.5.0",
    "eslint-import-resolver-typescript": "^2.7.1",
    "eslint-plugin-eslint-comments": "^3.2.0",
    "eslint-plugin-import": "^2.26.0",
    "eslint-plugin-prettier": "^4.2.1",
    "eslint-plugin-react-hooks": "^4.6.0",
    "lint-staged": "^13.0.3",
    prettier: "^2.7.1",
    "simple-git-hooks": "^2.8.1",
    tsup: "^6.2.3",
    typescript: "^4.8.4"
  },
  "simple-git-hooks": {
    "pre-commit": "pnpm lint-staged"
  },
  "lint-staged": {
    "*.{js,ts,tsx}": [
      "pnpm lint:fix"
    ],
    "*.{json,md,yml}": [
      "pnpm lint:format"
    ]
  },
  files: [
    "dist",
    "templates"
  ],
  repository: {
    type: "git",
    url: "https://github.com/wagmi-dev/wagmi.git"
  }
};

// src/templates.ts
var templates = [
  {
    name: "next",
    title: "Next.js",
    description: "Next.js wagmi project"
  },
  {
    name: "next-with-connectkit",
    title: "Next.js + ConnectKit",
    description: "Next.js wagmi project with ConnectKit included"
  },
  {
    name: "next-with-rainbowkit",
    title: "Next.js + RainbowKit",
    description: "Next.js wagmi project with RainbowKit included"
  },
  {
    name: "vite-react",
    title: "Vite",
    description: "Vite React wagmi project"
  },
  {
    name: "create-react-app",
    title: "Create React App",
    description: "Create React App wagmi project"
  }
];

// src/utils/getPackageManager.ts
import { detect } from "detect-package-manager";
var getPackageManager = ({ options: options2 }) => {
  if (options2) {
    if (options2.npm)
      return "npm";
    if (options2.pnpm)
      return "pnpm";
    if (options2.yarn)
      return "yarn";
  }
  const userAgent = process.env.npm_config_user_agent;
  if (userAgent) {
    if (userAgent.includes("pnpm"))
      return "pnpm";
    if (userAgent.includes("npm"))
      return "npm";
    if (userAgent.includes("yarn"))
      return "yarn";
  }
  return detect();
};

// src/utils/notifyUpdate.ts
import pico from "picocolors";
import checkForUpdate from "update-check";
var log = console.log;
async function notifyUpdate({ options: options2 }) {
  try {
    const res = await checkForUpdate(package_default);
    if (res == null ? void 0 : res.latest) {
      const packageManager = await getPackageManager({ options: options2 });
      const updateMessage = packageManager === "pnpm" ? "pnpm add -g create-wagmi" : packageManager === "yarn" ? "yarn global add create-wagmi" : "npm i -g create-wagmi";
      log(
        pico.bold(
          pico.yellow("A new version of `create-wagmi` is available!") + "\nYou can update by running: " + pico.cyan(updateMessage) + "\n"
        )
      );
    }
    process.exit();
  } catch {
  }
}

// src/utils/validate.ts
import fs from "fs-extra";
import pico2 from "picocolors";
import validatePackageName from "validate-npm-package-name";
import path from "path";
async function validateProjectName({
  projectName,
  projectPath
}) {
  const nameValidation = validatePackageName(projectName);
  if (!nameValidation.validForNewPackages) {
    const problems = [
      ...nameValidation.errors ?? [],
      ...nameValidation.warnings ?? []
    ];
    return {
      valid: false,
      message: `\u{1F648} "${projectName}" is not a valid project name.`,
      problems: problems.map((problem) => `\u{1F449} ${problem}`).join("\n")
    };
  }
  const targetPath = path.join(process.cwd(), projectPath);
  if (await fs.pathExists(targetPath))
    return {
      valid: false,
      message: `\u{1F648} the directory "${projectPath}" already exists.`,
      problems: "\u{1F449} choose another name or delete the directory."
    };
  return {
    valid: true
  };
}
async function validateTemplateName({
  isNameRequired = true,
  templateName,
  templatesPath
}) {
  if (isNameRequired && !templateName)
    return {
      valid: false,
      message: `\u{1F648} no template provided.`,
      problems: "\u{1F449} select a template or provide one using --template."
    };
  if (templateName && !await fs.pathExists(path.join(templatesPath, templateName)))
    return {
      valid: false,
      message: `\u{1F648} the template "${templateName}" does not exist.`,
      problems: `Choose a valid name. Available: ${templates.map(({ name }) => name).join(", ")}`
    };
  return { valid: true };
}
var ValidationError = class extends Error {
  name = "ValidationError";
  constructor(validation) {
    super([pico2.red(validation.message), validation.problems].join("\n"));
  }
};

// src/index.ts
import path2 from "path";
import { fileURLToPath } from "url";
var log2 = console.log;
var cli = cac(package_default.name).version(package_default.version).usage(`${pico3.green("<project-directory>")} [options]`).option(
  "-t, --template [name]",
  `A template to bootstrap with. Available: ${templates.map(({ name }) => name).join(", ")}`
).option("--npm", "Use npm as your package manager").option("--pnpm", "Use pnpm as your package manager").option("--yarn", "Use yarn as your package manager").option("--skip-git", "Skips initializing the project as a git repository").help();
var { args, options } = cli.parse(process.argv);
async function run() {
  var _a, _b;
  if (options.help)
    return;
  const __dirname = fileURLToPath(new URL(".", import.meta.url));
  const templatesPath = path2.join(__dirname, "..", "templates");
  let templateName = options.template || options.t;
  let templateValidation = await validateTemplateName({
    isNameRequired: false,
    templateName,
    templatesPath
  });
  if (!templateValidation.valid)
    throw new ValidationError(templateValidation);
  let projectName;
  let projectPath;
  if (args[0]) {
    projectPath = args[0].trim();
    const splitPath = projectPath.split("/");
    projectName = ((_a = splitPath[splitPath.length - 1]) == null ? void 0 : _a.trim()) || "";
    log2(pico3.green("\u2714"), pico3.bold(`Using project name:`), projectName);
  } else {
    const res = await prompts({
      initial: "my-app",
      name: "projectName",
      message: "What is your project named?",
      type: "text",
      async validate(projectName2) {
        const validation = await validateProjectName({
          projectName: projectName2,
          projectPath: projectName2
        });
        if (!validation.valid)
          return validation.message;
        return true;
      }
    });
    projectName = (_b = res.projectName) == null ? void 0 : _b.trim();
    projectPath = projectName;
  }
  const nameValidation = await validateProjectName({
    projectName,
    projectPath
  });
  if (!nameValidation.valid)
    throw new ValidationError(nameValidation);
  if (!templateName) {
    templateName = (await prompts({
      name: "templateName",
      message: "What template would you like to use?",
      type: "select",
      choices: templates.map(({ name, ...t }) => ({ ...t, value: name }))
    })).templateName;
  }
  templateValidation = await validateTemplateName({
    templateName,
    templatesPath
  });
  if (!templateValidation.valid)
    throw new ValidationError(templateValidation);
  const targetPath = path2.join(process.cwd(), projectPath);
  log2(`Creating a new wagmi app in ${pico3.green(targetPath)}.`);
  log2();
  const templatePath = path2.join(templatesPath, templateName);
  await cpy(path2.join(templatePath, "**", "*"), targetPath, {
    rename: (name) => name.replace(/^_dot_/, ".")
  });
  const packageJson = await fs2.readJSON(path2.join(targetPath, "package.json"));
  packageJson.name = projectName;
  await fs2.writeFile(
    path2.join(targetPath, "package.json"),
    JSON.stringify(packageJson, null, 2)
  );
  const packageManager = await getPackageManager({ options });
  log2(pico3.bold(`Using ${packageManager}.`));
  log2();
  log2("Installing packages. This might take a couple of minutes.");
  const installArgs = [
    "install",
    packageManager === "npm" ? "--quiet" : "--silent"
  ];
  await oraPromise(
    new Promise((resolve, reject) => {
      const child = execa(packageManager, installArgs, {
        cwd: targetPath,
        env: {
          ...process.env,
          ADBLOCK: "1",
          DISABLE_OPENCOLLECTIVE: "1",
          NODE_ENV: "development"
        },
        stdio: "inherit"
      });
      child.on("close", (code) => {
        if (code !== 0) {
          reject({ command: `${packageManager} ${installArgs.join(" ")}` });
          return;
        }
        resolve(true);
      });
    }),
    {
      failText: "Failed to install packages.",
      successText: "Installed packages."
    }
  );
  log2();
  if (!options.skipGit) {
    await execa("git", ["init"], { cwd: targetPath });
    await execa("git", ["add", "."], { cwd: targetPath });
    await execa(
      "git",
      [
        "commit",
        "--no-verify",
        "--message",
        "Initial commit from create-wagmi"
      ],
      { cwd: targetPath }
    );
    log2("Initialized a git repository.");
    log2();
  }
  log2(
    `${pico3.green("Success!")} Created ${projectName} at ${path2.resolve(
      projectPath
    )}`
  );
  log2();
  log2(
    `To start your app, run \`${pico3.bold(
      pico3.cyan(`cd ${projectPath}`)
    )}\` and then \`${pico3.bold(
      pico3.cyan(
        `${packageManager}${packageManager === "npm" ? " run" : ""} dev`
      )
    )}\``
  );
  log2();
}
(async () => {
  try {
    await run();
    await notifyUpdate({ options });
  } catch (error) {
    log2(
      error instanceof ValidationError ? error.message : pico3.red(error.message)
    );
    await notifyUpdate({ options });
    process.exit(1);
  }
})();
